@@ -24,7 +24,7 @@ struct client {
         char *uri_path;   //correspond to the registered uri_header 
         PyObject *wsgi_cb;
         int response_iter_sent; //-2: nothing sent, -1: header sent, 0-9999: iter sent
-        char response_header[MAXHEADER];
+        char *response_header;
         int response_header_length;
         PyObject *response_content;
         PyObject *response_content_obj;
