@@ -75,41 +75,6 @@ char * remove_leading_and_trailing_spaces(char* s)
 }
 
 
-/*
-This procedure append src to dest
-*/
-int str_append3(char *dest, char *src1, char *src2, char *src3, int n)
-{
-
-   int i=0;
-   int ret=1;
-
-   dest[0]='\0';
-   while (*src1!='\0' && ret==1)
-   {
-       *dest++=*src1++;
-       i++;
-       if (i>=n) ret=-1;
-   }
-   while (*src2!='\0' && ret==1)
-   {
-       *dest++=*src2++;
-       i++;
-       if (i>=n) ret=-2;
-   }
-   while (*src3!='\0' && ret==1)
-   {
-       *dest++=*src3++;
-       i++;
-       if (i>=n) ret=-3;
-   }
-   *dest='\0';  
-   if (ret==1) return ret;
-   else return i;
-}
-
-
-
 /*
 Provide a string representation of the current time
 */
