@@ -75,6 +75,7 @@ void close_connection(struct client *cli)
 {
     if (debug)
         printf("host=%s,port=%i close_connection:cli:%p, input_header:%p***\n",cli->remote_addr, cli->remote_port, cli, cli->input_header);
+    free(cli->remote_addr);
     free(cli->input_header);
     free(cli->cmd);
     free(cli->uri);
@@ -253,8 +254,8 @@ int python_handler(struct client *cli)
                PyString_Concat(&pydummy, PyString_FromString(", "));
         }
         PyString_Concat(&pydummy, PyString_FromString("\r\nContent-Length: 0\r\n\r\n"));
-        strcpy(cli->response_header,PyString_AsString(pydummy));
-	cli->response_header_length=strlen(cli->response_header);
+        cli->response_header = PyString_AsString(pydummy);
+	cli->response_header_length=(int)PyString_Size(pydummy);
         cli->response_content=PyList_New(0);
         Py_DECREF(pyenviron);
         return 1;
@@ -331,21 +332,26 @@ int python_handler(struct client *cli)
     if (cli->response_content!=NULL) 
     {
         PyObject *pydummy = PyObject_Str(pystart_response);
-        strcpy(cli->response_header,PyString_AsString(pydummy));
-	cli->response_header_length=strlen(cli->response_header);
+        cli->response_header = PyString_AsString(pydummy);
+	cli->response_header_length = (int)PyString_Size(pydummy);
         Py_DECREF(pydummy);
     }
     else 
     //python call return is NULL
     {
         printf("Python error!!!\n");
-        if (str_append3(cli->response_header,"HTTP/1.0 500 Not found\r\nContent-Type: text/html\r\nServer: ", VERSION, "\r\n\r\n", MAXHEADER)<0)
+        char buff[200];
+        sprintf(buff, "HTTP/1.0 500 Not found\r\nContent-Type: text/html\r\nServer: %s* \r\n\r\n", VERSION);
+        cli->response_header = buff;
+
+        if (cli->response_header == NULL)
         {
-             printf("ERROR!!!! Response header bigger than foreseen:%i", MAXHEADER);
-             printf("HEADER TOP\n%s\nHEADER BOT\n", cli->response_header);
-             return -1;
-        }
-	cli->response_header_length=strlen(cli->response_header);
+            printf("ERROR!!!! Memory allocation error in the Python error handling procedure\n");
+            cli->response_header_length=0;
+            goto leave_python_handler;
+        } 	
+        cli->response_header_length=strlen(cli->response_header);
+        
         if (PyErr_Occurred()) 
         { 
              //get_traceback();py_b
@@ -357,10 +363,6 @@ int python_handler(struct client *cli)
              PyObject *pysys=PyObject_GetAttrString(py_base_module,"sys");
              PyObject *pystderr=PyObject_GetAttrString(pysys,"stderr");
              Py_DECREF(pysys);
-/*             PyObject *pyclose_method=PyObject_GetAttrString(pystderr, "close");
-             PyObject *pyclose=PyObject_CallFunction(pyclose_method, NULL);
-             Py_DECREF(pyclose_method);
-             Py_DECREF(pyclose);*/
              PyObject *pygetvalue=PyObject_GetAttrString(pystderr, "getvalue");
              Py_DECREF(pystderr);
              PyObject *pyres=PyObject_CallFunction(pygetvalue, NULL);
@@ -393,6 +395,7 @@ int python_handler(struct client *cli)
              Py_DECREF(pydummy);
          }
     }
+leave_python_handler:
     Py_XDECREF(pystart_response);
     Py_XDECREF(pyenviron);
     return 1;
@@ -435,7 +438,6 @@ int write_cli(struct client *cli, char *response, size_t len,  int revents)
                     fprintf(stderr, "Connection closed after %i retries\n", cli->retry);
                     return 0; //stop the watcher and close the connection
                 }
-                // XXX We shouldn't sleep in an event-base server.
                 usleep(100000);  //failed but we don't want to close the watcher
             }
             if ((int)r==0)
@@ -463,7 +465,7 @@ This is the write call back registered within the event loop
 */
 void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
 { 
-    char response[MAXHEADER];
+    char response[1024];
     int stop=0; //0: not stop, 1: stop, 2: stop and call tp close
     int ret; //python_handler return
     struct client *cli= ((struct client*) (((char*)w) - offsetof(struct client,ev_write)));
@@ -474,27 +476,31 @@ void write_cb(struct ev_loop *loop, struct ev_io *w, int revents)
         if (ret==0) //look for python callback and execute it
         {
             //uri not found
-            str_append3(response,"HTTP/1.0 500 Not found\r\nContent-Type: text/html\r\nServer: ", VERSION ,"\r\n\r\n<html><head><title>Page not found</title></head><body><p>Page not found!!!</p></body></html>", MAXHEADER);
-            write_cli(cli,response, strlen(response), revents);
+            snprintf(response, sizeof(response), "HTTP/1.0 500 Not found\r\nContent-Type: text/html\r\nServer: %s\r\n\r\n<html><head><title>Page not found</title></head><body><p>Page not found!!!</p></body></html>", VERSION );
+            if (response) write_cli(cli,response, strlen(response), revents);
+            else printf("problem with error 500: Page not found\n");
             stop=1;
         } 
         else if (ret==-411)
         {
-            str_append3(response,"HTTP/1.0 411 Length Required\r\nContent-Type: text/html\r\nServer: ", VERSION, "\r\n\r\n<html><head><title>Length Required</head><body><p>Length Required!!!</p></body></html>", MAXHEADER);
-            write_cli(cli,response, strlen(response), revents);
+            snprintf(response, sizeof(response), "HTTP/1.0 411 Length Required\r\nContent-Type: text/html\r\nServer: %s\r\n\r\n<html><head><title>Length Required</head><body><p>Length Required!!!</p></body></html>", VERSION);
+            if (response) write_cli(cli,response, strlen(response), revents);
+            else printf("problem with error 411\n");
             stop=1;
         }
         else if (ret==-500)
         {
-            str_append3(response,"HTTP/1.0 500 Internal Server Error\r\nContent-Type: text/html\r\nServer: ", VERSION, "\r\n\r\n<html><head><title>Internal Server Error</title></head><body><p>Internal Server Error!!!</p></body></html>", MAXHEADER);
-            write_cli(cli,response, strlen(response), revents);
+            snprintf(response, sizeof(response), "HTTP/1.0 500 Internal Server Error\r\nContent-Type: text/html\r\nServer: %s\r\n\r\n<html><head><title>Internal Server Error</title></head><body><p>Internal Server Error!!!</p></body></html>", VERSION);
+            if (response) write_cli(cli,response, strlen(response), revents);
+            else printf("problem with error 500: Internal server error\n");
             stop=1;
         }
         else if (ret==-501)
         {
             //problem to parse the request
-            str_append3(response,"HTTP/1.0 501 Not Implemented\r\nContent-Type: text/html\r\nServer: ", VERSION, "\r\n\r\n<html><head><title>Not Implemented</head><body><p>Not Implemented!!!</p></body></html>", MAXHEADER);
-            write_cli(cli,response, strlen(response), revents);
+            snprintf(response,sizeof(response), "HTTP/1.0 501 Not Implemented\r\nContent-Type: text/html\r\nServer: %s\r\n\r\n<html><head><title>Not Implemented</head><body><p>Not Implemented!!!</p></body></html>", VERSION);
+            if (response) write_cli(cli,response, strlen(response), revents);
+            else printf("problem with error 501\n");
             stop=1;
         }
         else
@@ -744,7 +750,7 @@ void accept_cb(struct ev_loop *loop, struct ev_io *w, int revents)
     cli->cmd=NULL;
     cli->uri_path=NULL;
     cli->wsgi_cb=NULL;
-    cli->response_header[0]='\0';
+    cli->response_header = NULL;
     cli->response_content=NULL;
     cli->response_content_obj=NULL;
     if (debug)
@@ -752,8 +758,8 @@ void accept_cb(struct ev_loop *loop, struct ev_io *w, int revents)
     cli->input_pos=0;
     cli->retry=0;
     cli->response_iter_sent=-2;
-    cli->remote_addr=inet_ntoa (client_addr.sin_addr);
-    cli->remote_port=ntohs(client_addr.sin_port);
+    cli->remote_addr = strdup(inet_ntoa(client_addr.sin_addr));
+    cli->remote_port = ntohs(client_addr.sin_port);
     if (setnonblock(cli->fd) < 0)
                 fprintf(stderr, "failed to set client socket to non-blocking");
     ev_io_init(&cli->ev_read,connection_cb,cli->fd,EV_READ);
