@@ -9,9 +9,9 @@
 #include "common.h"
 #include "wsgi.h"
 
-char *server_name;
-char *server_port;
-int debug;
+extern char *server_name;
+extern char *server_port;
+extern int debug;
 
 /*
 This procdure analyse uri and return a Python dictionary with every parameters as keys and their associated values into a list. 
@@ -94,7 +94,8 @@ a dictionary key and list of values
  'fapws.raw_header': '...'
  'fapws.major_minor': '1.1', 
  'fapws.remote_addr': '127.0.0.1', 
- 'fapws.remote_port': 60580, 
+ 'fapws.remote_port': 60580,
+ 'fapws.socket_fd': 4,
  'REQUEST_METHOD': 'GET', 
  'SCRIPT_NAME': '/hello', 
  'QUERY_STRING': 'param=key', 
@@ -317,8 +318,6 @@ PyObject * header_to_dict (struct client *cli)
          break;
      }
    }
-   data[j]='\0';
-   //printf("BODY:%s***\n",data);
 
    free(data);
    free(buf2);
@@ -398,6 +397,9 @@ PyObject *py_get_request_info(struct client *cli)
     pydummy=Py_BuildValue("H", cli->remote_port);
     PyDict_SetItemString(pydict, "fapws.remote_port", pydummy);
     Py_DECREF(pydummy);
+    pydummy=PyLong_FromLong(cli->fd);
+    PyDict_SetItemString(pydict, "fapws.socket_fd", pydummy);
+    Py_DECREF(pydummy);
     return pydict;
 }
 
